/*
 *  Mixer two mp_image.
 *
 *  Copyright (C) 2007 Attila Ötvös <attila@onebithq.com>
 *
 *  MPlayer is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  MPlayer is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with MPlayer; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */


/**
 * \brief imgsize() returns the mpi planes size
 * \param mpi: mp_image_t pointer
 * \return full planes size
 */
int imgsize(mp_image_t *mpi);

/**
 * \brief imgscalex() returns the mpi planes[0] bytes per pixel
 * \param mpi: mp_image_t pointer
 * \return planes[0] bytes per pixel
 */
int imgscalex(mp_image_t *mpi);

/**
 * \brief cut_image() cut part image from src to dst
 * \param src: source image
 * \param dst: destination image
 * \param x,y: cut top right corner
 * \param w,h: cut image part width and height
 */
void cut_image(mp_image_t* src,mp_image_t* dst, int x, int y, int w, int h);

/**
 * \brief put_rawimage() mixing mpi image to dmpi image
 * \param dmpi : destination image
 * \param mpi  : source image
 * \param window : target window on dmpi (x,y: positon, w,h: size)
 * \param scaled : mpi size (w,h) and position (x,y)
 * \param alpha: insert image alpha
 */
void mp_image_mixer(mp_image_t* dmpi,
                    mp_image_t *mpi,
                    int window_x, int window_y, int window_w, int window_h,
                    int scaled_x, int scaled_y, int scaled_w, int scaled_h,
                    int alpha);
