
#include "config.h"

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>

#include "m_option.h"

#include "stream/stream.h"
#include "demuxer.h"
#include "stheader.h"

#include "libmpcodecs/img_format.h"

int blank_v_format = 0;
int blank_v_size_id = 0;
int blank_v_width = 0;
int blank_v_height = 0;
int blank_v_imgsize = 0;
int blank_v_aspect = 0;
int blank_v_color = 0;
float blank_v_fps = 25.0;

int blank_a_rate = 0;
int blank_a_channels = 2;
int blank_a_samplesize = 2;
int blank_a_bitrate = 0;

const m_option_t demux_blank_opts[] = {
    // size:
    { "w", &blank_v_width, CONF_TYPE_INT,CONF_RANGE,1,8192, NULL },
    { "h", &blank_v_height, CONF_TYPE_INT,CONF_RANGE,1,8192, NULL },
    { "sqcif", &blank_v_size_id, CONF_TYPE_FLAG,0,0,1, NULL },
    { "qcif", &blank_v_size_id, CONF_TYPE_FLAG,0,0,2, NULL },
    { "cif", &blank_v_size_id, CONF_TYPE_FLAG,0,0,3, NULL },
    { "4cif", &blank_v_size_id, CONF_TYPE_FLAG,0,0,4, NULL },
    { "pal", &blank_v_size_id, CONF_TYPE_FLAG,0,0,5, NULL },
    { "ntsc", &blank_v_size_id, CONF_TYPE_FLAG,0,0,6, NULL },
    { "16cif", &blank_v_size_id, CONF_TYPE_FLAG,0,0,7, NULL },
    { "sif", &blank_v_size_id, CONF_TYPE_FLAG,0,0,8, NULL },
    // format:
    { "format", &blank_v_format, CONF_TYPE_IMGFMT, 0, 0 , 0, NULL },
    // below options are obsolete
    { "i420", &blank_v_format, CONF_TYPE_FLAG, 0, 0 , IMGFMT_I420, NULL },
    { "yv12", &blank_v_format, CONF_TYPE_FLAG, 0, 0 , IMGFMT_YV12, NULL },
    { "nv12", &blank_v_format, CONF_TYPE_FLAG, 0, 0 , IMGFMT_NV12, NULL },
    { "hm12", &blank_v_format, CONF_TYPE_FLAG, 0, 0 , IMGFMT_HM12, NULL },
    { "yuy2", &blank_v_format, CONF_TYPE_FLAG, 0, 0 , IMGFMT_YUY2, NULL },
    { "uyvy", &blank_v_format, CONF_TYPE_FLAG, 0, 0 , IMGFMT_UYVY, NULL },
    { "y8", &blank_v_format, CONF_TYPE_FLAG, 0, 0 , IMGFMT_Y8, NULL },
    // misc:
    { "fps", &blank_v_fps, CONF_TYPE_FLOAT,CONF_RANGE,0.001,1000, NULL },
    { "color", &blank_v_color, CONF_TYPE_STRING, 0, 0 , 0, NULL },
    // audio:
    {"rate", &blank_a_rate, CONF_TYPE_INT, CONF_RANGE, 1000, 8*48000, NULL},
    {"bitrate", &blank_a_bitrate, CONF_TYPE_INT, CONF_RANGE, 1000, 8*48000, NULL},
    {"channels", &blank_a_channels, CONF_TYPE_INT, CONF_RANGE, 1, 6, NULL},
    {"nosound", &blank_a_channels, CONF_TYPE_FLAG, 0, 0, 0, NULL},

    {NULL, NULL, 0, 0, 0, 0, NULL}
};

typedef struct demux_blank_priv_st {
    int packet_size;
    float vpts;
    float apts;
    float apps;
    off_t vpos;
    off_t apos;
} demux_blank_priv_t;

static demuxer_t* demux_blank_open(demuxer_t* demuxer) {
    demux_blank_priv_t *priv;
    sh_video_t* sh_video;
    sh_audio_t* sh_audio;
    WAVEFORMATEX* w;

    if(!blank_v_width || !blank_v_height){
        mp_msg(MSGT_DEMUX,MSGL_ERR,"blank: width or height not specified!\n");
        return 0;
    }

    if(!blank_v_imgsize){
        mp_msg(MSGT_DEMUX,MSGL_ERR,
               "blank: img size not specified and unknown format!\n");
        return 0;
    }

    priv=calloc(1,sizeof(demux_blank_priv_t));

    sh_video = new_sh_video(demuxer,0);
    sh_video->format=blank_v_format;
    sh_video->fps=blank_v_fps;
    sh_video->frametime=1.0/sh_video->fps;
    sh_video->disp_w=blank_v_width;
    sh_video->disp_h=blank_v_height;
    sh_video->i_bps=blank_v_fps*blank_v_imgsize;

    demuxer->movi_start = demuxer->stream->start_pos;
    demuxer->movi_end = demuxer->stream->end_pos;

    demuxer->video->sh = sh_video;
    sh_video->ds = demuxer->video;

    if(demuxer->audio && demuxer->audio->id==-2)
        blank_a_channels=0;
    if(blank_a_channels) {
        if(!blank_a_rate)
            blank_a_rate=48000;
        sh_audio = new_sh_audio(demuxer,0);
        sh_audio->wf = w = malloc(sizeof(WAVEFORMATEX));
        w->wFormatTag = sh_audio->format = 0x01;
        w->nChannels = sh_audio->channels = blank_a_channels;
        w->nSamplesPerSec = sh_audio->samplerate = blank_a_rate;
        if (blank_a_bitrate > 999)
            w->nAvgBytesPerSec = blank_a_bitrate/8;
        else if (blank_a_bitrate > 0)
            w->nAvgBytesPerSec = blank_a_bitrate*125;
        else
            w->nAvgBytesPerSec = blank_a_rate*blank_a_samplesize*
                                 blank_a_channels;
        w->nBlockAlign = blank_a_channels*blank_a_samplesize;
        sh_audio->samplesize = blank_a_samplesize;
        w->wBitsPerSample = 8*blank_a_samplesize;
        w->cbSize = 0;

        demuxer->movi_start = demuxer->stream->start_pos;
        demuxer->movi_end = demuxer->stream->end_pos;

        demuxer->audio->sh = sh_audio;
        sh_audio->ds = demuxer->audio;

        priv->apps=1.0;
        priv->packet_size=blank_a_rate*blank_a_channels*2;
    }

    demuxer->priv=priv;
    return demuxer;
}

static int demux_blank_fill_buffer(demuxer_t* demuxer, demux_stream_t *ds) {
    demux_blank_priv_t *priv = demuxer->priv;
    demux_packet_t* dp;
    sh_video_t* sh_video;
    sh_audio_t* sh_audio;

    if(demuxer->stream->eof) return 0;
    if(ds==demuxer->video) {
        sh_video=demuxer->video->sh;
        ds_read_packet(ds,demuxer->stream,blank_v_imgsize,priv->vpts,
                       priv->vpos,0x10);
        priv->vpts+=sh_video->frametime;
        priv->vpos+=1;
        return 1;
    }
    if(ds==demuxer->audio) {
        sh_audio=demuxer->audio->sh;
        dp=new_demux_packet(priv->packet_size);
        memset(dp->buffer,0,priv->packet_size);
        resize_demux_packet(dp, priv->packet_size);
        dp->pts=priv->apts;
        dp->pos=priv->apos;
        dp->flags=0x10;
        ds_add_packet(demuxer->audio,dp);
        priv->apts+=priv->apps;
        priv->apos+=1;
        return 1;
    }
    return 0;
}

static void demux_blank_close(demuxer_t *demuxer) {
    demux_blank_priv_t *priv = demuxer->priv;

    if(priv) free(priv);
}


demuxer_desc_t demuxer_desc_blank = {
    "blank demuxer",
    "blank",
    "blank",
    "?",
    "",
    DEMUXER_TYPE_BLANK,
    1, // no autodetect
    NULL,
    demux_blank_fill_buffer,
    demux_blank_open,
    demux_blank_close,
    NULL,
    NULL
};
