/*
 *  Import video config parser
 *
 *  Based on vf.c
 *
 *  Copyright (C) 2007 Attila Ötvös <attila@onebithq.com>
 *
 *  MPlayer is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  MPlayer is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with MPlayer; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>


#include "config.h"

#include "mp_msg.h"
#include "m_config.h"
#include "m_option.h"
#include "m_struct.h"
#include "parser-mecmd.h"

#include "parser-meimport.h"

extern m_obj_list_t vf_obj_list;

typedef struct import_files_s {
    int id;
    m_entry_t* filelist;
    void *next;
} import_files_t;

id_import_config_t id_import_config_dflts = {
    -1,
    0,
    0.0,
    { .type = END_AT_NONE, .pos = 0 },
    0.0,
    0,
    0,
    1.0,
    0.0,
    0,

    -1,
    -1,
    -2,

    NULL,

    { 1, NULL }
};

typedef struct import_config_s {
    int id;
    id_import_config_t *config;
} import_config_t;

#define ST_OFF(f) M_ST_OFF(id_import_config_t,f)
static m_option_t id_import_config_opts_fields[] = {
    // number of frames to play/convert
    {"frames", ST_OFF(play_n_frames_mf), CONF_TYPE_INT, CONF_MIN, 0, 0, NULL},
    // seek to byte/seconds position
    {"sb", ST_OFF(seek_to_byte), CONF_TYPE_POSITION, CONF_MIN, 0, 0, NULL},
    {"ss", ST_OFF(seek_to_sec), CONF_TYPE_TIME, 0, 0, 0, NULL},
    // stop at given position
    {"endpos", ST_OFF(end_at), CONF_TYPE_TIME_SIZE, 0, 0, 0, NULL},
    // select audio/video/subtitle stream
    {"aid", ST_OFF(audio_id), CONF_TYPE_INT, CONF_RANGE, 0, 8190, NULL},
    {"vid", ST_OFF(video_id), CONF_TYPE_INT, CONF_RANGE, 0, 8190, NULL},
    {"sid", ST_OFF(dvdsub_id), CONF_TYPE_INT, CONF_RANGE, 0, 8190, NULL},
    {"novideo", ST_OFF(video_id), CONF_TYPE_FLAG, 0, -1, -2, NULL},

    {"fps", ST_OFF(force_fps), CONF_TYPE_FLOAT, CONF_MIN, 0, 0, NULL},
    // force video/audio rate:
    {"srate", ST_OFF(force_srate),CONF_TYPE_INT,CONF_RANGE,1000,8*48000,NULL},
    {"format", ST_OFF(audio_output_format), CONF_TYPE_AFMT, 0, 0, 0, NULL},
    {"speed", ST_OFF(playback_speed), CONF_TYPE_FLOAT,
     CONF_RANGE, 0.01, 100.0, NULL},
    // set a-v distance
    {"delay", ST_OFF(audio_delay), CONF_TYPE_FLOAT,
     CONF_RANGE, -100.0, 100.0, NULL},
    // ignore header-specified delay (dwStart)
    {"ignore-start", ST_OFF(ignore_start), CONF_TYPE_FLAG, 0, 0, 1, NULL},
    {"noignore-start", ST_OFF(ignore_start), CONF_TYPE_FLAG, 0, 1, 0, NULL},
    // disable audio
    {"sound", ST_OFF(audio_id), CONF_TYPE_FLAG, 0, -2, -1, NULL},
    {"nosound", ST_OFF(audio_id), CONF_TYPE_FLAG, 0, -1, -2, NULL},
    // video filters
    {"vf", ST_OFF(vf_settings), CONF_TYPE_OBJ_SETTINGS_LIST,0,0,0,&vf_obj_list},
    // audio filters
    {"af", ST_OFF(af_cfg.list), CONF_TYPE_STRING_LIST, 0, 0, 0, NULL},
    {"af-list", ST_OFF(af_cfg.list), CONF_TYPE_STRING_LIST, 0, 0, 0, NULL},
    {"af-force", ST_OFF(af_cfg.force), CONF_TYPE_INT, CONF_RANGE, 0, 7, NULL},

    { NULL, NULL, 0, 0, 0, 0,  NULL }
};
static struct m_struct_st import_config_opts = {
    "import-config",
    sizeof(id_import_config_t),
    &id_import_config_dflts,
    id_import_config_opts_fields
};

import_files_t *import_files = NULL;
import_config_t *import_config = NULL;

static int parse_import_config(int id, char *opt, char *param) {
    import_config_t *tmp = import_config;
    int nf = 0, cf = 0, r;
    id_import_config_t *config;

    if (tmp)
        while (tmp[nf].config) {
            if (tmp->id==id) cf=nf+1;
        nf++;
        }
    if (nf && !cf) {
            import_config=realloc(import_config,(nf+2)*sizeof(import_config_t));
        memset(&import_config[nf+1],0,sizeof(import_config_t));
        memset(&import_config[nf+2],0,sizeof(import_config_t));
        import_config[nf+1].id=id;
        import_config[nf+1].config=malloc(sizeof(id_import_config_t));
        config=import_config[nf+1].config;
        *(config)=id_import_config_dflts;
        cf=nf+1;
    }
    else if (!nf) {
            import_config = malloc(2*sizeof(import_config_t));
        memset(import_config,0,2*sizeof(import_config_t));
        import_config[0].id=id;
        import_config[0].config=malloc(sizeof(id_import_config_t));
        config=import_config[0].config;
        *(config)=id_import_config_dflts;
        cf=1;
    }
    else if (nf && cf)
        config=import_config[cf-1].config;

    r = m_struct_set( &import_config_opts, config, opt, param);

    return r;
}

static char* parserid(int *id, const char *param){
    char *tmp=strdup(param),*p,*ret;
    if(!(p=strchr(tmp,':'))) {
        free(tmp);
        return NULL;
    }
    *p++=0;
    if((*id=atoi(tmp))<=0) {
        free(tmp);
        return NULL;
    }
    ret=strdup(p);
    free(tmp);
return ret;
}

static import_files_t* add_import_files(int id){
    import_files_t *tmp = import_files;
    while (tmp) {
        if (tmp->id==id) return tmp;
        if (!tmp->next) {
            tmp->next=malloc(sizeof(import_files_t));
            tmp=tmp->next;
            memset(tmp,0,sizeof(import_files_t));
            tmp->id=id;
            return tmp;
        }
        tmp=tmp->next;
    }
    import_files=malloc(sizeof(import_files_t));
    memset(import_files,0,sizeof(import_files_t));
    import_files->id=id;
    return import_files;
}

static m_entry_t* filename_parser(m_entry_t* lst, const char *param){
    int nf=0;
    char *p;

    if (lst)
        free(lst);
    lst = calloc(1,sizeof(m_entry_t));
    while (param){
        if((p=strchr(param,','))) *p++=0;
        lst = realloc(lst,(nf+2)*sizeof(m_entry_t));
        lst[nf].name = strdup(param);
        lst[nf].opts = calloc(2,sizeof(char*));
        memset(&lst[nf+1],0,sizeof(m_entry_t));
        nf++;
        param=p;
    }
    return lst;
}

int import_parseoption(const m_option_t* conf,
                       const char *opt,
                       const char *param){
    int id=-1, r;
    import_files_t *insfn;
    char *tmp, *confname, *ps, *pe;

    if (!strcmp(opt,"import-files")) {
        if (!(tmp=parserid(&id,param)))
            return ERR_MISSING_PARAM;
            insfn=add_import_files(id);
        insfn->filelist=filename_parser(insfn->filelist,tmp);
        free(tmp);
        return 1;
    }
    confname=opt+7;
    if (!(tmp=parserid(&id,param)))
        return ERR_MISSING_PARAM;
    if (id==0) return ERR_OUT_OF_RANGE;
    r=parse_import_config(id, confname, tmp);
    free(tmp);
    if(r>0) return 1;
    return ERR_NOT_AN_OPTION;
}

int import_files_count(void){
    int n = 0;
    import_files_t *tmp = import_files;

    while (tmp) {tmp=tmp->next; n++;}
    return n;
}

int import_files_num2id(int num){
    int n = 0;
    import_files_t *tmp = import_files;

    while (tmp) {
            if(n==num) return tmp->id;
        tmp=tmp->next;
        n++;
    }
    return 0;
}

void* import_files_id(int id){
    import_files_t *tmp = import_files;
    while (tmp) if (tmp->id==id) return tmp->filelist; else tmp=tmp->next;
    return NULL;
}

id_import_config_t* import_config_id(int id){
    import_config_t *tmp = import_config;
    id_import_config_t *config = NULL;
    int nl = 0;

    if(!id) return NULL;
    if(tmp)
        while (tmp[nl].config)
            if (tmp[nl].id==id)
                return tmp[nl].config;
            else
                nl++;
    config=malloc(sizeof(id_import_config_t));
    *(config)=id_import_config_dflts;
    return config;
}

void* importdummyfilelist(void){

    m_entry_t* ret = calloc(1,sizeof(m_entry_t));
    ret->name = strdup("blank://");
    ret->opts = calloc(2,sizeof(char*));
    return ret;
}
